@@ -1,4 +1,4 @@
-/* $OpenBSD: ssh-dss.c,v 1.24 2006/11/06 21:25:28 markus Exp $ */
+/* $OpenBSD: ssh-dss.c,v 1.25 2010/02/26 20:29:54 djm Exp $ */
 /*
  * Copyright (c) 2000 Markus Friedl.  All rights reserved.
  *
@@ -50,7 +50,9 @@ ssh_dss_sign(const Key *key, u_char **sigp, u_int *lenp,
 	u_int rlen, slen, len, dlen;
 	Buffer b;
 
-	if (key == NULL || key->type != KEY_DSA || key->dsa == NULL) {
+	if (key == NULL ||
+	    (key->type != KEY_DSA && key->type != KEY_DSA_CERT) ||
+	    key->dsa == NULL) {
 		error("ssh_dss_sign: no DSA key");
 		return -1;
 	}
@@ -113,7 +115,9 @@ ssh_dss_verify(const Key *key, const u_char *signature, u_int signaturelen,
 	int rlen, ret;
 	Buffer b;
 
-	if (key == NULL || key->type != KEY_DSA || key->dsa == NULL) {
+	if (key == NULL ||
+	    (key->type != KEY_DSA && key->type != KEY_DSA_CERT) ||
+	    key->dsa == NULL) {
 		error("ssh_dss_verify: no DSA key");
 		return -1;
 	}
